@@ -19,10 +19,12 @@
  */
 
 #include "application.h"
+#include <libgnome/gnome-help.h> //For gnome_help_display
 #include "config.h" //For VERSION.
 #include <cstdio>
 #include <libintl.h>
 
+
 App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
 : Gtk::Window(cobject), //It's a virtual base class, so we have to call the specific constructor to prevent the default constructor from being called.
   type_base(cobject, "Glom"),
@@ -169,20 +171,19 @@ void App_Glom::init_menus()
     "        <menuitem action='GlomAction_Menu_Navigate_Database' />"
     "        <menuitem action='GlomAction_Menu_Navigate_Table' />"
     "     </menu>"
-    "      <menu action='Glom_Menu_UserLevel'>"
-    "        <menuitem action='GlomAction_Menu_UserLevel_Developer' />"
-    "        <menuitem action='GlomAction_Menu_UserLevel_Operator' />"
-    "      </menu>"
     "      <menu action='Glom_Menu_Mode'>"
     "        <menuitem action='GlomAction_Menu_Mode_Data' />"
     "        <menuitem action='GlomAction_Menu_Mode_Find' />"
     "      </menu>"
+    "      <menu action='Glom_Menu_UserLevel'>"
+    "        <menuitem action='GlomAction_Menu_UserLevel_Developer' />"
+    "        <menuitem action='GlomAction_Menu_UserLevel_Operator' />"
+    "      </menu>"
     "      <menu action='Glom_Menu_Developer'>"
     "        <menuitem action='GlomAction_Menu_Developer_Fields' />"
     "        <menuitem action='GlomAction_Menu_Developer_Relationships' />"    
+    "        <menuitem action='GlomAction_Menu_Developer_Layout' />"
     "        <menuitem action='GlomAction_Menu_Developer_Users' />"
-    "        <menuitem action='GlomAction_Menu_Developer_Layout' />"           
-    "         <placeholder name='Glom_Menu_Developer_PH' />"
     "      </menu>"
     "    </placeholder>"
     "  </menubar>"
@@ -257,7 +258,7 @@ void App_Glom::on_document_load()
 
       //Switch to operator mode when opening new documents:
       pDocument->set_userlevel(AppState::USERLEVEL_OPERATOR);
-            
+
       m_pFrame->do_menu_Navigate_Database(false); //false = don't show list, just open the current database after connecting.
     }
   }
@@ -343,6 +344,36 @@ void App_Glom::set_mode_data()
   m_action_mode_data->activate();
 }
 
+
+void App_Glom::init_menus_help()
+{
+  //Call base class:
+  App_WithDoc_Gtk::init_menus_help();
+  m_refHelpActionGroup->add( Gtk::Action::create("BakeryAction_Help_Contents",
+                        gettext("_Contents"), gettext("Help with the application")),
+                        sigc::mem_fun(*this, &App_Glom::on_menu_help_contents) );
+
+  //Build part of the menu structure, to be merged in by using the "PH" plaeholders:
+  static const Glib::ustring ui_description =
+    "<ui>"
+    "  <menubar name='Bakery_MainMenu'>"
+    "    <placeholder name='Bakery_MenuPH_Help'>"
+    "      <menu action='BakeryAction_Menu_Help'>"
+    "        <menuitem action='BakeryAction_Help_Contents' />"
+    "      </menu>"
+    "    </placeholder>"
+    "  </menubar>"
+    "</ui>";
+
+  //Add menu:
+  add_ui_from_string(ui_description);
+}
+
+void App_Glom::on_menu_help_contents()
+{
+  gnome_help_display("glom", 0, 0);
+}
+
 AppState::userlevels App_Glom::get_userlevel() const
 {
   const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
@@ -356,3 +387,7 @@ AppState::userlevels App_Glom::get_userlevel() const
 
 
 
+
+
+
+
