@@ -20,6 +20,7 @@
 
 //#include <gnome.h>
 #include <gtkmm/main.h>
+#include <libgnome/gnome-init.h> // For gnome_program_init().
 #include "config.h" //For VERSION.
 
 #include "application.h"
@@ -37,6 +38,10 @@ main(int argc, char* argv[])
 
   try
   {
+    //Initialize gnome_program, so that we can use gnome_help_display().    
+    gnome_program_init (PACKAGE, VERSION, LIBGNOME_MODULE, argc, argv,
+                            GNOME_PROGRAM_STANDARD_PROPERTIES, 0);
+                            
     Gtk::Main mainInstance(argc, argv);
     Bakery::init();
 
