@@ -46,8 +46,6 @@ Box_Tables::Box_Tables(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   
   refGlade->get_widget("checkbutton_show_hidden", m_pCheckButtonShowHidden);
   m_pCheckButtonShowHidden->signal_toggled().connect(sigc::mem_fun(*this, &Box_Tables::on_show_hidden_toggled));
-   
-  m_AddDel.set_show_row_titles(false);
   
   m_AddDel.signal_user_added().connect(sigc::mem_fun(*this, &Box_Tables::on_AddDel_Add));
   m_AddDel.signal_user_requested_delete().connect(sigc::mem_fun(*this, &Box_Tables::on_AddDel_Delete));
@@ -61,6 +59,18 @@ Box_Tables::~Box_Tables()
 {
 }
 
+void Box_Tables::fill_table_row(const Gtk::TreeModel::iterator& iter, const TableInfo& table_info)
+{
+  if(iter)
+  {
+    m_AddDel.set_value_key(iter, table_info.m_name);
+    m_AddDel.set_value(iter, m_colTableName, table_info.m_name);
+    m_AddDel.set_value(iter, m_colHidden, table_info.m_name);
+    m_AddDel.set_value(iter, m_colTitle, table_info.m_title);
+    m_AddDel.set_value(iter, m_colDefault, table_info.m_default);
+  }
+}
+  
 void Box_Tables::fill_from_database()
 {
   Bakery::BusyCursor(*get_app_window());
@@ -78,7 +88,6 @@ void Box_Tables::fill_from_database()
   //Add the columns:
   m_AddDel.remove_all_columns();
 
-  //TODO: Show the title instead of the name when in operator mode
   bool editable = developer_mode;
   m_colTableName = m_AddDel.add_column(gettext("Tables"), AddDelColumnInfo::STYLE_Text, editable);
 
@@ -94,10 +103,10 @@ void Box_Tables::fill_from_database()
 
   //Get the list of hidden tables:
 
-  Document_Glom::type_listTableInfo listTables;
+  Document_Glom::type_listTableInfo listTablesDocument;
   if(m_pDocument)
   {
-    listTables = m_pDocument->get_tables();
+    listTablesDocument = m_pDocument->get_tables();
   }
   else
     g_warning("debug: m_pDocument is null");
@@ -113,35 +122,41 @@ void Box_Tables::fill_from_database()
 
     for(type_vecStrings::iterator iter = vecTables.begin(); iter != vecTables.end(); iter++)
     {
-      const Glib::ustring& strName = *iter;
+      const Glib::ustring strName = *iter;
 
       //Check whether it should be hidden:
-      Document_Glom::type_listTableInfo::iterator iterFind = std::find_if(listTables.begin(), listTables.end(), predicate_FieldHasName<TableInfo>(strName));
-      bool hidden = false;
-      Glib::ustring title;
-      bool is_default = false;
-      if(iterFind != listTables.end())
+      Document_Glom::type_listTableInfo::iterator iterFind = std::find_if(listTablesDocument.begin(), listTablesDocument.end(), predicate_FieldHasName<TableInfo>(strName));
+      if(iterFind != listTablesDocument.end())
       {
-        hidden = iterFind->m_hidden;
-        title = iterFind->m_title;
-        is_default = iterFind->m_default;
-      }
+        bool hidden = iterFind->m_hidden;
 
-      bool bAddIt = true;
-      if(hidden && !developer_mode)  //Don't add hidden tables unless we are in developer mode:
-        bAddIt = false;
+        bool bAddIt = true;
+        if(hidden && !developer_mode)  //Don't add hidden tables unless we are in developer mode:
+          bAddIt = false;
 
-      if(hidden && !m_pCheckButtonShowHidden->get_active()) //Don't add hidden tables if that checkbox is unset.
-        bAddIt = false;
+        if(hidden && !m_pCheckButtonShowHidden->get_active()) //Don't add hidden tables if that checkbox is unset.
+          bAddIt = false;
 
-      if(bAddIt)
-      {
-          guint uiRow = m_AddDel.add_item(strName);
-
-          m_AddDel.set_value(uiRow, m_colHidden, hidden);
-          m_AddDel.set_value(uiRow, m_colTitle, title);
-          m_AddDel.set_value(uiRow, m_colDefault, is_default);
+        if(bAddIt)
+        {
+          Gtk::TreeModel::iterator iter = m_AddDel.add_item(strName);
+          fill_table_row(iter, *iterFind);
         }
+      }
+      else
+      {
+        //This table is in the database, but not in the document.
+        //Show it as hidden:
+        if(developer_mode)
+        {
+          TableInfo table_info;
+          table_info.m_name = strName;
+          table_info.m_hidden = true;
+          
+          Gtk::TreeModel::iterator iter = m_AddDel.add_item(strName);
+          fill_table_row(iter, table_info);
+        }
+      }
     }
   }
 
@@ -153,53 +168,62 @@ void Box_Tables::fill_from_database()
   m_modified = false;
 }
 
-void Box_Tables::on_AddDel_Add(guint row)
+void Box_Tables::on_AddDel_Add(const Gtk::TreeModel::iterator& row)
 {
-  Glib::ustring table_name = m_AddDel.get_value(row);
+  Glib::ustring table_name = m_AddDel.get_value(row, m_colTableName);
+  if(!table_name.empty())
+  {
+    Glib::ustring primary_key_name =   table_name + "_id";
 
-  Glib::ustring primary_key_name =   table_name + "_id";
-  
-  //Create a table with 1 "ID" field:
-  #if 0 //MSYQL:
-  Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " INT NOT NULL AUTO_INCREMENT PRIMARY KEY)" );
-  Query_execute( "INSERT INTO " + table_name + " VALUES (0)" );
-  #else
-  //PostgresSQL:
-  //Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " serial NOT NULL  PRIMARY KEY)" );
-  Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " numeric NOT NULL  PRIMARY KEY)" );
+    //Create a table with 1 "ID" field:
+   //MSYQL:
+    //Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " INT NOT NULL AUTO_INCREMENT PRIMARY KEY)" );
+    //Query_execute( "INSERT INTO " + table_name + " VALUES (0)" );
 
-  //Save the field information directly into the database, because we can not get all the correct information from the database.
-  //Otherwise some information would be forgotten:
-  Field field_primary_key;
-  Gnome::Gda::FieldAttributes field_info = field_primary_key.get_field_info();
-  field_info.set_name(primary_key_name);
-  field_info.set_primary_key();
-  field_info.set_auto_increment();
-  field_info.set_allow_null(false);
-  field_primary_key.set_field_info(field_info);
-  field_primary_key.set_glom_type(Field::TYPE_NUMERIC);
-  
-  type_vecFields fields;
-  fields.push_back(field_primary_key);
+    //PostgresSQL:
+    //Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " serial NOT NULL  PRIMARY KEY)" );
+    Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " numeric NOT NULL  PRIMARY KEY)" );
 
-   if(m_pDocument)
-      m_pDocument->set_table_fields(table_name, fields);
-  
-  //Query_execute( "INSERT INTO " + strName + " VALUES (0)" );
-  #endif
+    //Show the new information for this whole row:
+    TableInfo table_info;
+    table_info.m_name = table_name;
+    fill_table_row(row, table_info);
+    
+    //Save the field information directly into the database, because we can not get all the correct information from the database.
+    //Otherwise some information would be forgotten:
+    Field field_primary_key;
+    Gnome::Gda::FieldAttributes field_info = field_primary_key.get_field_info();
+    field_info.set_name(primary_key_name);
+    field_info.set_primary_key();
+    field_info.set_auto_increment();
+    field_info.set_allow_null(false);
+    field_primary_key.set_field_info(field_info);
+    field_primary_key.set_glom_type(Field::TYPE_NUMERIC);
 
-  save_to_document();
-  fill_from_database();
+    type_vecFields fields;
+    fields.push_back(field_primary_key);
 
-  m_modified = true;
+     if(m_pDocument)
+        m_pDocument->set_table_fields(table_name, fields);
+
+    save_to_document();
+    //fill_from_database(); //We should not modify the model structure in a cellrenderer signal handler.
+
+    m_modified = true;
+  }
 }
 
-void Box_Tables::on_AddDel_Delete(guint rowStart, guint rowEnd)
+void Box_Tables::on_AddDel_Delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& rowEnd)
 {
-  for(guint row = rowStart; row <= rowEnd; row++)
+  Gtk::TreeModel::iterator iterAfter = rowEnd;
+  ++iterAfter;
+
+  bool something_changed = false;
+  for(Gtk::TreeModel::iterator iter = rowStart; iter != iterAfter; ++iter)
   {
-    Glib::ustring strName = m_AddDel.get_value(row);
-    if(strName.size())
+    Glib::ustring strName = m_AddDel.get_value_key(iter);
+
+    if(!strName.empty())
     {
       //Ask the user to confirm:
       Glib::ustring strMsg = gettext("Are you sure that you want to delete this table?\nTable name: ")
@@ -211,20 +235,24 @@ void Box_Tables::on_AddDel_Delete(guint rowStart, guint rowEnd)
       if(iButtonClicked == Gtk::RESPONSE_OK)
       {
         Query_execute( "DROP TABLE " + strName);
+        something_changed = true;
       }
     }
   }
 
-  save_to_document();
-  
-  fill_from_database();
+  if(something_changed)
+  {
+    save_to_document();
 
-  m_modified = true;
+    fill_from_database();
+
+    m_modified = true;
+  }
 }
 
-void Box_Tables::on_AddDel_Edit(guint row)
+void Box_Tables::on_AddDel_Edit(const Gtk::TreeModel::iterator& row)
 {
-  Glib::ustring table_name = m_AddDel.get_value(row);
+  Glib::ustring table_name = m_AddDel.get_value_key(row);
   
   if(m_pDocument)
   {
@@ -257,14 +285,13 @@ void Box_Tables::save_to_document()
       //Save the hidden tables. TODO_usermode: Only if we are in developer mode.
       Document_Glom::type_listTableInfo listTables;
 
-      guint rows_count = m_AddDel.get_count();
-      for(guint row_index = 0; row_index < rows_count; ++row_index)
+      for(Gtk::TreeModel::iterator iter = m_AddDel.get_model()->children().begin(); iter != m_AddDel.get_model()->children().end(); ++iter)
       {
         TableInfo table_info;
-        table_info.m_name = m_AddDel.get_value(row_index, m_colTableName);
-        table_info.m_hidden  = m_AddDel.get_value_as_bool(row_index, m_colHidden);
-        table_info.m_title  = m_AddDel.get_value(row_index, m_colTitle);
-        table_info.m_default  = m_AddDel.get_value_as_bool(row_index, m_colDefault);
+        table_info.m_name = m_AddDel.get_value(iter, m_colTableName);
+        table_info.m_hidden  = m_AddDel.get_value_as_bool(iter, m_colHidden);
+        table_info.m_title  = m_AddDel.get_value(iter, m_colTitle);
+        table_info.m_default  = m_AddDel.get_value_as_bool(iter, m_colDefault);
 
         listTables.push_back(table_info);
       }
@@ -280,7 +307,7 @@ void Box_Tables::on_show_hidden_toggled()
   fill_from_database();
 }
 
-void Box_Tables::on_AddDel_changed(guint /* row TODO */, guint column)
+void Box_Tables::on_AddDel_changed(const Gtk::TreeModel::iterator& row, guint column)
 {
   if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
   {
@@ -297,8 +324,22 @@ void Box_Tables::on_AddDel_changed(guint /* row TODO */, guint column)
     } 
     else if(column == m_colTableName)
     {
-      g_warning("Box_Tables: renaming of tables is not yet implemented."); //TODO
-      fill_from_database();
+      Glib::ustring table_name = m_AddDel.get_value_key(row);
+      Glib::ustring table_name_new = m_AddDel.get_value(row, m_colTableName);
+      if(!table_name.empty() && !table_name_new.empty())
+      {
+        Glib::ustring strMsg = gettext("Are you sure that you want to rename this table?");  //TODO: Show old and new names?
+        Gtk::MessageDialog dialog(strMsg);
+        int iButtonClicked = dialog.run();
+
+        //Rename the table:
+        if(iButtonClicked == Gtk::RESPONSE_OK)
+        {
+          Query_execute( "ALTER TABLE " + table_name + " RENAME TO " + table_name_new);
+          set_modified();
+          //fill_from_database(); //We should not modify the model structure in a cellrenderer signal handler.
+        }
+      }
     }
   }
 }
